TeamReview = TeamReview or {}

if TeamReview.Achievements then return end

local thisDir
function TeamReview:LoadLoc()
	local thisPath = debug.getinfo(2, "S").source:sub(2)
	thisDir = string.match(thisPath, '.*/')
	local upDir = thisDir:match('(.*/).-/')

	local lang = 'en'
	if not TeamReview.settings.language or TeamReview.settings.language == 1 then
		if BLT and BLT.Localization and BLT.Localization.get_language then
			lang = BLT.Localization:get_language()
			lang = lang.language or 'en'
		end
		
		if file.DirectoryExists('PD2TL/') then
			lang = 'th'
		end
		
		if lang == 'cht' or lang == 'zh-cn' then
			lang = 'cn'
		end
	else
		if TeamReview.languages[TeamReview.settings.language] then
			lang = TeamReview.languages[TeamReview.settings.language]
		end
	end

	local locFile = upDir..'loc/'..lang..'.lua'
	local f,err = io.open(locFile, 'r')
	if f then
		f:close()
		dofile(locFile)
	else
		lang = 'en'
		locFile = upDir..'loc/'..lang..'.lua'
		dofile(locFile)
	end
	
	TeamReview.language = lang
end
TeamReview:LoadLoc()

local function Loc(str_id, default)
	if TeamReview.settings.messages
		and TeamReview.settings.messages[str_id] ~= nil
		and not TeamReview.settings.messages[str_id]
	then
		return ''
	end
		
	if TeamReview.loc and TeamReview.loc[str_id] then
		return TeamReview.loc[str_id]
	end
	return default or str_id
end


local Cl = {
				good = Color(0,1,0),
				critical = Color(1,0,0),
				warning = Color(1,1,0.2),
				info = Color(0.7,0.7,0.7),
				achievement = Color(0.2,0.2,1),
				special = Color(0.8,0.3,0.9)
			}

TeamReview.alerts = {}
TeamReview.alerts_new = {}
TeamReview.outfits = {}

TeamReview.update_interval = 0.06

function TeamReview:AddAlert(text_id, color, str, position)

	if TeamReview.settings then
		if TeamReview.settings[color] ~= nil
			and not TeamReview.settings[color]
		then
			return
		end
		
		if str == TeamReview.me_inx
			and TeamReview.settings['review_me'] ~= nil
			and not TeamReview.settings['review_me']
		then
			return
		end
		
		if TeamReview.settings.messages
			and TeamReview.settings.messages[text_id] ~= nil
			and not TeamReview.settings.messages[text_id]
		then
			return
		end
	end

	color = Cl[color] or Color(1,1,1)
	
	local text = Loc(text_id)
	if str then
		text = string.format(text, str)
	end
	
	if text_id:match('.saw') or text_id:match('c4') then
		local levelTwists = TeamReview.levels[TeamReview.level] or TeamReview.levels[TeamReview.level..'_1'] or {}
		local sets = TeamReview.settings.levels[TeamReview.level] or {}
		if sets['saw_c4_comment'] ~= false then
			local id = sets['saw_c4_comment_custom'] or levelTwists['saw_c4_comment'] or ''
			text = text .. Loc(id, id)
		end
	end

	if position then
		if position <= #TeamReview.alerts_new then
			for i=position, #TeamReview.alerts_new do
				TeamReview.alerts_new[i+1] = TeamReview.alerts_new[i]
			end
		else
			position = #TeamReview.alerts_new + 1
		end
		
		TeamReview.alerts_new[position] = {text = text, color = color}
	else
		TeamReview.alerts_new[#TeamReview.alerts_new + 1] = {text = text, color = color}
	end
end

function TeamReview:Review()

	local CS = managers.crime_spree and managers.crime_spree.is_active()
	TeamReview.CS = CS

	TeamReview.outfits = {}
	TeamReview.host_name = nil
	TeamReview.num_of_players = 1
	
	if managers.network
		and managers.network:session()
		and managers.network:session():peers()
	then
		if HuskPlayerMovement and not managers.network:session():local_peer():synched() then
			TeamReview.alerts = {{text = Loc('joining'), color = Cl['info'] or Color(1,1,1)}}
			return
		end
	
		for i,p in pairs(managers.network:session():peers()) do
			local inx = p:name()
			while TeamReview.outfits[inx] do
				inx = inx..p:id()
			end
			
			TeamReview.outfits[inx] = p:blackmarket_outfit()
			TeamReview.outfits[inx].level = p:level() or 0
			
			-- Apparently, #TeamReview.outfits does not work
			TeamReview.num_of_players = TeamReview.num_of_players + 1
			
			if p:is_host() then
				TeamReview.host_name = inx
			end
		end
	else
		return
	end
	
	local me = managers.network:session():local_peer()
	local inx = me:name()
	while TeamReview.outfits[me:name()] do
		inx = inx..'.'
	end
	
	TeamReview.me_inx = inx
	
	-- duh
	TeamReview.outfits[inx] =  managers.blackmarket:unpack_outfit_from_string(managers.blackmarket:outfit_string())
	
	TeamReview.outfits[inx].level = managers.experience:current_level() or 0
	if not TeamReview.host_name then
		TeamReview.host_name = inx
	end
	
	TeamReview.counter = TeamReview.counter or 1
	local funcs = {
		TeamReview.Weapons,
		TeamReview.Deployables,
		TeamReview.Skills,
		TeamReview.Achievements,
		TeamReview.Assets,
		TeamReview.PreJoin
	}
	
	funcs[TeamReview.counter]()
	
	TeamReview.counter = TeamReview.counter + 1
	if TeamReview.counter > #funcs then
		TeamReview.counter = 1
		
		if #TeamReview.alerts_new == 0 then
			TeamReview:AddAlert('all_good', 'good')
		end
		
		if TeamReview.update_available then
			TeamReview:AddAlert('updates', 'special', nil, 50)
		end
		
		TeamReview.alerts = TeamReview.alerts_new
		TeamReview.alerts_new = {}
	end
end

function TeamReview:Weapons()
	local weapons = {}
	local issues = {}
	
	for k,v in pairs(TeamReview.outfits) do
		
		weapons[k] = { v.primary, v.secondary }
		issues[k] = {
						both_silent = true,
						both_shotguns = true,
						both_pistols = true,
						both_no_pickup = true,
						both_loud = true
					}
	end
	
	local skirmish = managers.skirmish and managers.skirmish:is_skirmish() or false
	TeamReview.skirmish = skirmish
	
	local present_categories = {}
	local saw_present = false
	local db_or_he_present = false
	
	local anti_winters_present = false
	TeamReview.anti_winters = TeamReview.anti_winters or {}
	
	for k,v in pairs(weapons) do
		
		local this_both_silent = true
		local this_both_loud = true
		local this_both_shotguns = true
		local this_both_pistols = true
		local this_both_no_pickup = true
		
		for k2,v2 in pairs(v) do
			
			-- Please implement more IDs!
			local td = tweak_data.weapon[managers.weapon_factory:get_weapon_id_by_factory_id(v2.factory_id)]
			
			if td and td.categories then
				
				local this_shotgun = false
				local this_pistol = false
				local this_anybow = false
				local this_bow_explosive = false
				
				for k3,v3 in pairs(td.categories) do
					present_categories[v3] = true
					
					if v3 == 'shotgun' then
						this_shotgun = true
						for k4,v4 in pairs(v2.blueprint) do
							if v4:match('dragons_breath') then
								this_shotgun = false -- DB does not require skills
								db_or_he_present = true
								break --k4v4
							end
							
							if v4:match('explosive') then
								db_or_he_present = true
							end
						end
					end
					
					if v3 == 'bow' or v3 == 'crossbow' then
						this_anybow = true
						for k4,v4 in pairs(v2.blueprint) do
							if v4:match('explosion')
								or v4:match('explosive')
							then
								db_or_he_present = true
								this_bow_explosive = true
							end
						end
						
						this_both_no_pickup = this_both_no_pickup and this_bow_explosive or false
					end
					
					if v3 == 'pistol' then
						this_pistol = true
					end
					
					if v3 == 'saw' then
						saw_present = true
						if skirmish then
							TeamReview:AddAlert('saw_skirmish', 'critical', k)
						end
					end
				end
				
				if not this_pistol then
					this_both_pistols = false
				end
				
				if not this_shotgun then
					this_both_shotguns = false
				end
			end
			
			if managers.weapon_factory:has_perk("silencer", v2.factory_id, v2.blueprint)
				or (this_anybow and not this_bow_explosive)
			then
				this_both_loud = false
			else
				this_both_silent = false
			end
			
			if td and td.AMMO_PICKUP
				and not this_anybow
			then
				if td.AMMO_PICKUP[1] ~= 0 or td.AMMO_PICKUP[2] ~= 0 then
					this_both_no_pickup = false
				end
			end
			
			if TeamReview.anti_winters[v2.factory_id] then
				anti_winters_present = true
			end
		end
	
		issues[k].both_pistols = this_both_pistols
		issues[k].both_no_pickup = this_both_no_pickup
		issues[k].both_silent = this_both_silent
		issues[k].both_shotguns = this_both_shotguns
		issues[k].both_loud = this_both_loud
	end
	
	
	local level = managers.job:current_level_id() or ''
	TeamReview.level = level
	
	local levelTwists = {}
	for k,v in pairs(TeamReview.levels[level] or TeamReview.levels[level..'_1'] or TeamReview.levels[level:gsub('_night$', '')] or {}) do
		levelTwists[k] = v
	end
	levelTwists.has_winters = levelTwists.has_winters and managers.job:current_difficulty_stars() >= 2
	for k,v in pairs(TeamReview.settings.levels[level] or TeamReview.settings.levels[level..'_1'] or TeamReview.settings.levels[level:gsub('_night$', '')] or {}) do
		levelTwists[k] = v
	end
	
	local stealth = managers.job:is_level_ghostable_required(level) or levelTwists.stealth or false
	if skirmish then
		stealth = false
	end
	TeamReview.stealth = stealth
	
	local c4_present = 0
	
	
	-- If host has a stealth loadout then we must be doing it in stealth
	TeamReview.host_name = TeamReview.host_name or ''
	if not stealth and not skirmish
		and managers.job:is_level_ghostable(level) -- not an obstacle for a noob host though
		and not issues[TeamReview.host_name].both_loud
	then
		local perk = tonumber(TeamReview.outfits[TeamReview.host_name].skills.specializations[1]) or 0
		local stealth_depl = 'ecm_jammer bodybags_bag'
		
		if perk == 7
			or stealth_depl:match(TeamReview.outfits[TeamReview.host_name].deployable or '000')
			or stealth_depl:match(TeamReview.outfits[TeamReview.host_name].secondary_deployable or '000')
		then
			-- host uses stealth build
			stealth = true
			TeamReview.stealth = true
			
			TeamReview:AddAlert('seems_stealth', 'info')
		end
	end
	--------------------
	
	
	
	for k,v in pairs(issues) do
		if v.both_pistols and not stealth then
			local sp = tonumber(TeamReview.outfits[k].skills.skills[13]) or 0
			if sp < 12 then
				-- uses pistols without pistol skills
				TeamReview:AddAlert('pistols_no_skills', 'warning', k)
			end
		end
		
		if v.both_shotguns and not stealth then
			local sp = tonumber(TeamReview.outfits[k].skills.skills[4]) or 0
			if sp < 12 then
				-- uses shotguns without shotgun skills
				TeamReview:AddAlert('shotguns_no_skills', 'warning', k)
			end
		end
		
		if v.both_no_pickup then
				-- uses both no pickup
				TeamReview:AddAlert('both_no_pickup', 'critical', k)
		end
		
		if not v.both_loud and not stealth then
			local perk = tonumber(TeamReview.outfits[k].skills.specializations[1]) or 0
			local stealth_depl = 'ecm_jammer bodybags_bag'
			
			if perk == 7
				or stealth_depl:match(TeamReview.outfits[k].deployable or '000')
				or stealth_depl:match(TeamReview.outfits[k].secondary_deployable or '000')
			then
				-- uses stealth build
				TeamReview:AddAlert('stealth_build', 'critical', k)
			end
		end
		
		if v.both_loud and stealth then
			-- does not have a silenced weapon
			TeamReview:AddAlert('no_silencer', 'critical', k)
		end
		
		
		if (TeamReview.outfits[k].deployable or '000') == 'trip_mine' then
			local amnt = TeamReview.outfits[k].deployable_amount
			c4_present = c4_present + (amnt==3 and 3 or (amnt==7 and 4 or 6))
		elseif (TeamReview.outfits[k].secondary_deployable or '000') == 'trip_mine' then
			local amnt = TeamReview.outfits[k].secondary_deployable_amount
			c4_present = c4_present + (amnt==2 and 2 or (amnt==4 and 2 or 3))
		end
	end
	
	local thrown_saw_or_c4 = false
	if (levelTwists.requires_saw_or_c4
			or (not stealth and levelTwists.requires_saw_or_c4_loud))
		and not skirmish
		and not saw_present
		and c4_present == 0
	then
		-- c4 or saw would be useful here
		TeamReview:AddAlert('need_c4_or_saw', 'info')
		thrown_saw_or_c4 = true
	else
		if levelTwists.requires_c4
			and not (levelTwists.requires_saw_or_c4_loud and stealth)
			and not skirmish
		then
			if c4_present == 0 then
				-- c4 would be useful here
				TeamReview:AddAlert('need_c4', 'info')
			elseif levelTwists.requires_6_c4 and c4_present < 6 then
				-- too little c4
				TeamReview:AddAlert('more_c4', 'warning')
			end
		end
	end
	
	if not skirmish then
		if saw_present then
			local got_ammo = false
			for k,v in pairs(TeamReview.outfits) do
				if (v.deployable or '000') == 'ammo_bag'
					or (v.secondary_deployable or '000') == 'ammo_bag'
				then
					got_ammo = true
					break
				end
			end
			
			if not got_ammo then
				-- no ammo for the saw
				TeamReview:AddAlert('saw_ammo', 'critical')
			end
		elseif not thrown_saw_or_c4
				and (levelTwists.requires_saw
					or (not stealth and levelTwists.requires_saw_loud))
		then
			-- a saw would be useful here
			TeamReview:AddAlert('need_saw', 'info')
		end
	end
	
	if not stealth and not skirmish
		and levelTwists.has_winters
		and not (db_or_he_present
				or present_categories['grenade_launcher']
				or present_categories['flamethrower']
				or (TeamReview.settings
					and TeamReview.settings['snipers_winters']
					and present_categories['snp'])
				or anti_winters_present)
	then
		-- no good weapons for winters
		TeamReview:AddAlert('no_weap_winters', 'critical')
	end
end

function TeamReview:Deployables()
	local doctor_bag_present = false
	for k,v in pairs(TeamReview.outfits) do
	
		if (v.deployable or '000') == 'doctor_bag'
			or (v.secondary_deployable or '000') == 'doctor_bag'
		then
			doctor_bag_present = true
		end
		
		if (v.deployable or '000') == 'first_aid_kit'
			and v.deployable_amount < 7
		then
			-- brings 4 faks
			TeamReview:AddAlert('has_4_faks', 'warning', k)
		end
		
		if string.match(v.deployable or '000', 'sentry_gun') then
			local sp = tonumber(TeamReview.outfits[k].skills.skills[7]) or 0
			if sp < 15 then
				-- uses sentries without sentry skills
				TeamReview:AddAlert('sentry_no_skills', 'warning', k)
			end
		end
		
		if (v.deployable or '000') == 'armor_kit' then
			-- brings armor bag
			TeamReview:AddAlert('armor_bag', 'warning', k)
		end
	end
	
	if not doctor_bag_present and not TeamReview.stealth then
		-- no doctor bags
		TeamReview:AddAlert('no_meds', 'critical')
	end
end

function TeamReview:Skills()
	local difficulty = managers.job:current_difficulty_stars()
	if TeamReview.CS then
		local CS_level = managers.crime_spree:server_spree_level() or 0
		if CS_level > 299 then
			difficulty = difficulty + 3
		else
			difficulty = difficulty + math.floor(CS_level / 100)
		end
	end
	
	local level = managers.job:current_level_id() or ''
	local stealth = managers.job:is_level_ghostable(level) or true
	
	TeamReview.level = TeamReview.level or ''
	local levelTwists = {}
	for k,v in pairs(TeamReview.levels[level] or TeamReview.levels[level..'_1'] or TeamReview.levels[level:gsub('_night$', '')] or {}) do
		levelTwists[k] = v
	end
	for k,v in pairs(TeamReview.settings.levels[level] or TeamReview.settings.levels[level..'_1'] or TeamReview.settings.levels[level:gsub('_night$', '')] or {}) do
		levelTwists[k] = v
	end
	
	local inspire_present = 0
	local maniac_present = 0
	local drills_present = false
	
	local skillpoints = {}
	for k,v in pairs(TeamReview.outfits) do
		skillpoints[k] = {}
		local total_skillpoints = 0
		for i=1, 15 do
			skillpoints[k][i] = tonumber(v.skills.skills[i] or 0) or 0
			total_skillpoints = total_skillpoints + skillpoints[k][i]
		end
		
		if not HuskPlayerMovement and total_skillpoints < v.level + math.floor(v.level/10) * 2 then
			-- has unspent skill points
			TeamReview:AddAlert('unspent_points', 'info', k)
		end
		
		if total_skillpoints > 120 then
			-- cheated skillpoints
			TeamReview:AddAlert('cheated_points', 'critical', k)
		end
		
		-- Yes, there ARE such players out there
		if skillpoints[k][1]
			and skillpoints[k][1] < 6
			and skillpoints[k][6] < 6
			and skillpoints[k][7] < 6
			and skillpoints[k][8] < 12
			and skillpoints[k][10] < 12
		then
			-- does not have an upgraded deployable
			TeamReview:AddAlert('no_deployable', 'warning', k)
		end
	end
		
		
	for k,v in pairs(TeamReview.outfits) do
	
		local sp = skillpoints[k][1] or 0
		if sp > 27 then
			inspire_present = inspire_present + 1
		end
		
		sp = skillpoints[k][14] or 0
		if not TeamReview.stealth and Global.game_settings.one_down and sp < 4 then
			-- does not have Nine Lives aced on OD
			TeamReview:AddAlert('no_nine_lives', 'critical', k)
		end
		
		-- "Stealth?"
		sp = skillpoints[k][10] or 0
		if not stealth and sp >= 15 then
			-- stealth skills on a loud heist
			TeamReview:AddAlert('stealth_skills_on_loud', 'critical', k)
		end
		
		sp = skillpoints[k][7] or 0
		if not TeamReview.stealth and sp >= 15 then
			if not (string.match(v.deployable or '000', 'sentry_gun')
				or string.match(v.secondary_deployable or '000', 'sentry_gun'))
			then
				-- uses sentry skills without sentries
				TeamReview:AddAlert('no_sentry_w_skills', 'warning', k)
			end
		end
		
		sp = skillpoints[k][8] or 0
		if sp > 2 then
			drills_present = true
		end
		
		-- No idea which one of them is actually used
		local armor = math.max(tonumber(v.armor:sub(-1)) or 1,
							tonumber(v.armor_current:sub(-1)) or 1,
							tonumber(v.armor_current_state:sub(-1)) or 1
						)
		local perk = tonumber(v.skills.specializations[1]) or 0
		if perk == 14 then
			maniac_present = maniac_present + 1
		elseif not TeamReview.stealth and perk == 1 and difficulty > 2 then
			-- crew chief
			TeamReview:AddAlert('crew_chief', 'critical', k)
		elseif not TeamReview.stealth and perk == 6 and (armor == 1 or armor > 4) then
			-- wrong crook armor
			TeamReview:AddAlert('crook_armor', 'critical', k)
		elseif not TeamReview.stealth and perk == 11 and armor > 2 then
			-- wrong grinder armor
			TeamReview:AddAlert('grinder_armor', 'critical', k)
		end
	end
	
	if not TeamReview.stealth
		and difficulty > 3
		and TeamReview.num_of_players == 1
		and not Global.game_settings.team_ai
		and skillpoints[TeamReview.host_name]
		and skillpoints[TeamReview.host_name][14] < 12
	then
		-- should get more revenant skills playing solo
		TeamReview:AddAlert('more_revenant', 'warning')
	end
	
	if not TeamReview.stealth
		and difficulty > 3
		and inspire_present == 0
		and TeamReview.num_of_players > 1
	then
		-- no inspire
		TeamReview:AddAlert('no_inspire', 'critical')
	end
	
	if not TeamReview.stealth 
		and difficulty > 5 
		and inspire_present == 1 
		and TeamReview.num_of_players > 2
	then
		-- not enough inspire
		TeamReview:AddAlert('not_enough_inspire', 'critical')
	end
	
	if not TeamReview.stealth  and maniac_present > 1 then
		-- more than one player with maniac
		TeamReview:AddAlert('many_maniacs', 'critical')
	end
	
	if (levelTwists.drills
			or (not TeamReview.stealth and levelTwists.drills_loud))
		and not drills_present
	then
		-- no drill skills
		TeamReview:AddAlert('no_drills', 'info')
	end
	
	if not TeamReview.stealth then
		local weapons = {}
		
		for k,v in pairs(TeamReview.outfits) do
			weapons[k] = { v.primary, v.secondary }
		end
		
		for k,v in pairs(weapons) do
		
			local got_a_shotgun = false
			local got_a_pistol = false
		
			for k2,v2 in pairs(v) do
				
				local td = tweak_data.weapon[managers.weapon_factory:get_weapon_id_by_factory_id(v2.factory_id)]
				
				if td and td.categories then	
					for k3,v3 in pairs(td.categories) do
							got_a_shotgun = got_a_shotgun or v3 == 'shotgun'
							got_a_pistol = got_a_pistol or v3 == 'pistol'
					end
				else
					got_a_shotgun = true
					got_a_pistol = true
				end
			end
			
			if not got_a_shotgun then
				local sp = tonumber(TeamReview.outfits[k].skills.skills[4]) or 0
				if sp > 10 then
					-- no shotgun with shotgun skills
					TeamReview:AddAlert('no_shotgun_w_skills', 'warning', k)
				end
			end
			
			if not got_a_pistol then
				local sp = tonumber(TeamReview.outfits[k].skills.skills[13]) or 0
				if sp > 10 then
					-- no pistol with pistol skills
					TeamReview:AddAlert('no_pistol_w_skills', 'warning', k)
				end
			end
		end
	end
end

function TeamReview:Achievements()
	local hostMask = TeamReview.outfits[TeamReview.host_name] or nil
	hostMask = hostMask and hostMask.mask.mask_id
	for k,v in pairs(tweak_data.achievement.four_mask_achievements) do
		for k2,v2 in pairs(v.masks) do
			if v2 == hostMask then
				local job_pass = false
				local diff_pass = false
				
				if v.jobs then
					local curJob = managers.job:current_job_id() or ''
					for k3,v3 in pairs(v.jobs) do
						if curJob == v3 then
							job_pass = true
							break
						end
					end
				else
					job_pass = true
				end
				
				if v.difficulties then
					local curDiff = managers.job:current_difficulty_stars()
					if 7 - curDiff <= #v.difficulties then
						diff_pass = true
					end
				else
					diff_pass = true
				end
				
				if job_pass and diff_pass then
					-- we might be going for an achievement
					TeamReview:AddAlert('four_masks', 'achievement')
				end
			end
		end
	end
end

function TeamReview:Assets()
	if not HuskPlayerMovement or TeamReview.assetsBought then return end
	
	local peer = managers.network:session():local_peer()
	if peer
		and not peer:is_host()
		and not peer:synched()
	then
		return
	end
	
	local assetsBought = false
	local assetsAvailable = false
	
	if tweak_data.preplanning
		and tweak_data.preplanning.locations
		and tweak_data.preplanning.locations[TeamReview.level]
	then
		assetsBought = managers.preplanning._finished_preplan
		assetsAvailable = true
		for k,v in pairs(managers.preplanning._reserved_mission_elements or {}) do
			assetsBought = true
			break
		end
	elseif managers.assets then
		for k,v in pairs(managers.assets._global.assets or {}) do
			if v.show and v.can_unlock then
				if v.unlocked then
					assetsBought = true
					break
				else
					assetsAvailable = true
				end
			end
		end
	end
	
	TeamReview.assetsBought = assetsBought or nil
	
	if not assetsBought and assetsAvailable then
		-- No assets have been unlocked
		TeamReview:AddAlert('no_assets', 'warning')
	end
end

function TeamReview:PreJoin()
	local lp = managers.network:session():local_peer()
	if not HuskPlayerMovement
		or game_state_machine:current_state_name() == 'ingame_waiting_for_players'
		or lp:is_host()
		or lp:waiting_for_player_ready()
	then
		return
	end
	
	local everyone_down = true
	for i,p in pairs(managers.network:session():peers()) do
		local mvmnt = p:unit() and p:unit():movement() or nil
		if TeamReview.stealth then
			if p:is_host() then
				if mvmnt and mvmnt._suspicion_ratio then
					-- Not safe to join now
					TeamReview:AddAlert('not_safe', 'critical')
				end
				return
			end
		else
			if mvmnt and not mvmnt:downed() then
				everyone_down = false
				break
			end
		end
	end
	
	if everyone_down then
		-- Join and revive them now
		TeamReview:AddAlert('join_and_revive', 'critical', nil, 1)
	end
end