TeamReview = TeamReview or {}

TeamReview.settings = {
	good = true,
	critical = true,
	warning = true,
	info = true,
	achievement = true,
	review_me = true,
	crime_spree_lobby = true,
	pause_menu = true,
	snipers_winters = false,
	language = 1,
	background = true,
	messages = {},
	levels = {}
}

-- Just here, nowhere else
TeamReview.languages = { 'blt', 'en', 'ru', 'th', 'it', 'de', 'cn' }

local thisPath
local thisDir
local upDir
local function Dirs()
	thisPath = debug.getinfo(2, "S").source:sub(2)
	thisDir = string.match(thisPath, '.*/')
	upDir = thisDir:match('(.*/).-/')
end
Dirs()
Dirs = nil

-- If there is no BLT
if not MenuHelper then
	dofile(thisDir..'TeamReviewData.lua')
	dofile(thisDir..'TeamReview.lua')
	return
end

function TeamReview:LoadSettings()
	local file = io.open(SavePath .. 'Team Review.txt', "r")
	if file then
		for k, v in pairs(json.decode(file:read("*all")) or {}) do
			TeamReview.settings[k] = v
		end
		file:close()
	end
end
TeamReview.LoadSettings()

function TeamReview:SaveSettings()
	local file = io.open(SavePath .. 'Team Review.txt', "w")
	if file then
		file:write(json.encode(TeamReview.settings or {}))
		file:close()
	end
end

function TeamReview:LoadLocMenu()
	local lang = 'en'
	
	if not TeamReview.settings.language or TeamReview.settings.language == 1 then
		if BLT and BLT.Localization and BLT.Localization.get_language then
			lang = BLT.Localization:get_language()
			lang = lang.language or 'en'
		end
		
		if file.DirectoryExists('PD2TL/') then
			lang = 'th'
		end
		
		if lang == 'cht' or lang == 'zh-cn' then
			lang = 'cn'
		end
	else
		if TeamReview.languages[TeamReview.settings.language] then
			lang = TeamReview.languages[TeamReview.settings.language]
		end
	end
	
	local locFile = upDir .. 'menu/loc/'.. lang ..'.json'
	local f,err = io.open(locFile, 'r')
	if f then
		f:close()
	else
		lang = 'en'
		locFile = upDir .. 'menu/loc/'.. lang ..'.json'
	end
	LocalizationManager:load_localization_file(locFile)
	
	TeamReview.lang = lang
	
	dofile(upDir .. 'loc/lang_names.lua')
	if TeamReview.lang_names then
		LocalizationManager:add_localized_strings(TeamReview.lang_names)
	end
end
Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_team_review", TeamReview.LoadLocMenu)

function MenuCallbackHandler:tr_toggle_importance(item)
	local index = item._parameters.name
	if TeamReview.settings[index] ~= nil then
		TeamReview.settings[index] = item:value() == 'on' and true or false
	end
end

function MenuCallbackHandler:tr_toggle_message(item)
	local index = item._parameters.name
	if TeamReview.settings.messages then
		local val = item:value() == 'on'
		if val then
			TeamReview.settings.messages[index] = nil
		else
			TeamReview.settings.messages[index] = false
		end
	end
end

function MenuCallbackHandler:tr_toggle_level_param(item)
	local index = item._parameters.name
	local split = string.split(index, ':::')

	local val = item:value() == 'on'
	TeamReview.settings.levels[split[1]] = TeamReview.settings.levels[split[1]] or {}
	
	if val ~= (TeamReview.levels[split[1]][split[2]] and true or false) then
			TeamReview.settings.levels[split[1]][split[2]] = val
	else
		TeamReview.settings.levels[split[1]][split[2]] = nil
	end
	
	if not next(TeamReview.settings.levels[split[1]]) then
		TeamReview.settings.levels[split[1]] = nil
	end
end

function MenuCallbackHandler:tr_change_comment(item)
	local index = item._parameters.name
	local split = string.split(index, ':::')
	local val = item:value()
	
	TeamReview.settings.levels[split[1]] = TeamReview.settings.levels[split[1]] or {}
	local def = TeamReview.levels[split[1]]['saw_c4_comment']
	if val ~= '' and (not def or val ~= def) then
		TeamReview.settings.levels[split[1]]['saw_c4_comment_custom'] = val
	else
		TeamReview.settings.levels[split[1]]['saw_c4_comment_custom'] = nil
		item:set_value(def and TeamReview.loc[def] or '')
	end
	
	if not next(TeamReview.settings.levels[split[1]]) then
		TeamReview.settings.levels[split[1]] = nil
	end
end

function MenuCallbackHandler:tr_levels_status(item)
	local message = ''
	
	for k,v in pairs(TeamReview.settings.levels or {}) do
		message = message .. managers.localization:text('heist_'..k:gsub('_night$', ''):gsub('_skip1$', ''):gsub('peta_1', 'peta')..'_hl')
		message = message .. ' (' .. k .. ')\n'
		for k2,v2 in pairs(v) do
			if k2 == 'saw_c4_comment_custom' then
				message = message .. '    ' .. v2 .. '\n'
			else
				message = message .. '    ' .. managers.localization:text('tr_opt_' .. k2) .. ' = ' .. (v2 and 'true' or 'false') .. '\n'
			end
		end
		message = message .. '\n'
	end
	
	if message == '' then
		message = managers.localization:text('tr_opt_all_enabled')
	end
	
	QuickMenu:new(
					'Team Review', 
					message,
					 {{text = 'OK', is_cancel_button = true}}	
			):Show()
end

local function MakeMsgMenu()
	for k,v in pairs(TeamReview.loc or {}) do 
		local length = utf8.len(v)
		local val = TeamReview.settings.messages[k]
		val = val == nil and true or val
		local lim = ({ru=38,th=48,it=42})[TeamReview.lang] or 40
		MenuHelper:AddToggle({
								id = k,
								title = length > lim and utf8.sub(v,1,lim) or v,
								desc = v,
								callback = 'tr_toggle_message',
								value = val,
								default_value = true,
								menu_id = 'team_review_opt_msg',
								localized = false
							})
	end
	
	if not BLTSuperMod then
		local menu_msgs = MenuHelper:GetMenu('team_review_opt_msg')
		for k,v in pairs(menu_msgs._items_list or {}) do
			v:set_parameter('localize_help', false)
		end
	end
end

local function MakeLevelsMenu(nodes, first)
	if not first then
		local menu_levels = MenuHelper:GetMenu('team_review_opt_lvl')
		menu_levels._items_list = {}
		menu_levels:clean_items()
	end

	for k,v in pairs(TeamReview.levels or {}) do
		local menu_name = 'tr_levels_'..k
		
		if first then
			MenuHelper:NewMenu(menu_name)
		else
			local menu = MenuHelper:GetMenu(menu_name)
			menu._items_list = {}
			menu:clean_items()
		end
		
		for k2,v2 in pairs(TeamReview.levels_tweaks or v) do
			local val = v[k2] or false
			if TeamReview.settings.levels[k] then
				val = TeamReview.settings.levels[k][k2]
				val = val == nil and v[k2] or val
			end
			MenuHelper:AddToggle({
									id = k..':::'..k2,
									title = 'tr_opt_'..k2,
									desc = 'tr_opt_'..k2,
									callback = 'tr_toggle_level_param',
									value = val,
									default_value = true,
									menu_id = menu_name,
									localized = true
								})
		end
		
		local comment_def = TeamReview.settings.levels[k] and TeamReview.settings.levels[k]['saw_c4_comment_custom'] or nil
		comment_def = comment_def or (v['saw_c4_comment'] and TeamReview.loc[v['saw_c4_comment']] or '')
		MenuHelper:AddInput({
						id = k..':::comment',
						title = ' ',
						desc = '',
						callback = 'tr_change_comment',
						value = comment_def,
						menu_id = menu_name,
						priority = 15,
						localized = false
					})
	end
	
	local job_names_ids = {}
	for k,v in pairs(tweak_data.narrative:get_jobs_index()) do
		for k2,v2 in pairs(tweak_data.narrative:job_chain(v)) do
			if v2.level_id and TeamReview.levels[v2.level_id] then
				job_names_ids[v2.level_id] = (tweak_data.narrative:job_data(v) or {}).name_id
			end
		end
	end
	
	job_names_ids['branchbank'] = nil
	job_names_ids['escape_garage'] = nil
	job_names_ids['election_day_3_skip1'] = 'heist_election_day'
	job_names_ids['peta_1'] = 'heist_peta'
	
	MenuHelper:AddButton({
						id = 'tr_levels_status',
						title = managers.localization:text('tr_levels_status'),
						desc = '',
						callback = 'tr_levels_status',
						menu_id = 'team_review_opt_lvl',
						priority = 2,
						localized = false
					})
	MenuHelper:AddDivider({
					id = 'divider_team_review_opt_lvl_1',
					size = 20,
					menu_id = 'team_review_opt_lvl',
					priority = 1
				})
	
	for k,v in pairs(TeamReview.levels or {}) do
		local menu_name = 'tr_levels_'..k
		if first then
			nodes[menu_name] = MenuHelper:BuildMenu(menu_name)
		else
			MenuHelper:BuildMenu(menu_name)
		end
		local job_name = job_names_ids[k] and managers.localization:text(job_names_ids[k]) or nil
		local button_title = managers.localization:text('heist_'..k:gsub('_night$', ''):gsub('_skip1$', ''):gsub('peta_1', 'peta')..'_hl')

		if job_name and job_name ~= button_title then
			button_title = job_name .. ': ' .. button_title
		end
		
		local lim = (({ru=40})[TeamReview.lang] or 50)
		if utf8.len(button_title) > lim then
			button_title = utf8.sub(button_title,1,lim)
		end
		
		MenuHelper:AddButton({
						id = menu_name,
						title = button_title,
						desc = k,
						next_node = menu_name,
						menu_id = 'team_review_opt_lvl',
						localized = false
					})
	end
	
	MenuHelper:BuildMenu('team_review_opt_lvl')
end

function MenuCallbackHandler:tr_language(item)
	TeamReview.settings.language = item:value() or 1
	TeamReview:LoadLoc()
	TeamReview:LoadLocMenu()
	
	local menu_msgs = MenuHelper:GetMenu('team_review_opt_msg')
	menu_msgs._items_list = {}
	menu_msgs:clean_items()
	MakeMsgMenu()
	
	MakeLevelsMenu()
	
	MenuHelper:BuildMenu('team_review_opt_msg')
	
	managers.menu:back()
	managers.menu:open_node('team_review_opt')
end

function MenuCallbackHandler:tr_save()
	TeamReview.SaveSettings()
end

MenuHelper:LoadFromJsonFile(upDir .. 'menu/main.json', TeamReview, TeamReview.settings)
MenuHelper:LoadFromJsonFile(upDir .. 'menu/messages.json', TeamReview, TeamReview.settings.messages)
MenuHelper:LoadFromJsonFile(upDir .. 'menu/levels.json', TeamReview, TeamReview.settings.levels)

Hooks:Add("MenuManagerPopulateCustomMenus", "PopulateCustomMenus_TeamReview", function( menu_manager, nodes )
	dofile(thisDir..'TeamReviewData.lua')
	dofile(thisDir..'TeamReview.lua')
	
	local items = {}
	for k,v in pairs(TeamReview.languages) do
		items[k] = 'tr_lang_name_'..v
	end
	MenuHelper:AddMultipleChoice({
				id = 'language',
				title = 'LANGUAGE',
				desc = 'Set the preffered mod language',
				callback = 'tr_language',
				items = items,
				value = TeamReview.settings.language or 1,
				default_value = 1,
				menu_id = 'team_review_opt',
				localized = false,
				priority = 100
			})
			
	if not BLTSuperMod then
		local menu_settings = MenuHelper:GetMenu('team_review_opt')
		menu_settings._items_list[#menu_settings._items_list]:set_parameter('localize_help', false)
	end
	
	MakeMsgMenu()
end)

Hooks:Add("MenuManagerBuildCustomMenus", "BuildCustomMenus_TeamReview", function(menu_manager, nodes)
	MakeLevelsMenu(nodes, true)
end)