local oldTime = 0

local original_init = ContractBoxGui.init
function ContractBoxGui:init(...)
	original_init(self, ...)
	self:TeamReviewInit()
end

local original_update = ContractBoxGui.update
function ContractBoxGui:update(...)
	--original_update(self, ...)
	
	if not self._contract_text_header
		and managers.job:current_contact_data()
	then 
		self:refresh()
		return
	end
	
	for i = 1, tweak_data.max_players, 1 do
		self:update_character(i)
	end
	
	local apptime = Application:time()
	if apptime - oldTime > (TeamReview.update_interval or 0.1) 
		and TeamReview
		and TeamReview.Review
	then
		TeamReview:Review()
		oldTime = apptime
	end
	
	if self.TR and self.TR.review_text then 
		self:clear_review()
	end
		
	for k,v in pairs(TeamReview.alerts or {}) do
		self:add_review_line(v.text, v.color)
	end

	-- if not self.b then
		-- self.a = Overlay:newgui():create_screen_workspace() or {}
		-- self.b = self.a:panel():text{name = "team_review", x = - (1920/2.1) + 0.5 * 1920, y = - (1080/4) + 5/9 * 1080, text = "43251345", font = tweak_data.menu.pd2_small_font, font_size = tweak_data.menu.pd2_small_font_size, color = Color.red, layer = 1}
	-- end
end

function ContractBoxGui:clear_review()
	for i=1, #self.TR.review_text do
		self._contract_panel:remove(self.TR.review_text[i])
	end
	
	self.TR.review_text = {}
end

function ContractBoxGui:add_review_line(text, color)
	self.TR = self.TR or {}
	self.TR.review_text = self.TR.review_text or {}
	self.TR.review_text[#self.TR.review_text + 1] = self._contract_panel:text({
			blend_mode = "add",
			x = 7,
			y = 28 + (#self.TR.review_text * 22),
			font = tweak_data.menu.pd2_small_font,
			font_size = tweak_data.menu.pd2_small_font_size,
			color = color,
			text = text
		})
	local mul = #TeamReview.alerts > 0 and #TeamReview.alerts - 1 or 0
	self._contract_panel:set_h(100 + (mul * 20))
	self._contract_panel:set_rightbottom(self._panel:w() - 0, self._panel:h() - 60)
	
	if self._contract_text_header then
		self._contract_text_header:set_bottom(self._contract_panel:top())
		self._contract_text_header:set_left(self._contract_panel:left())
	end
end

function ContractBoxGui:TeamReviewInit()
	local thisPath = debug.getinfo(2, "S").source:sub(2)
	local thisDir = string.match(thisPath, '.*/')
	local upDir = thisDir:match('(.*/).-/')
	dofile(thisDir..'TeamReviewData.lua')
	dofile(thisDir..'TeamReview.lua')
	dofile(upDir..'Updates.lua')
end

function ContractBoxGui:refresh()
	if self._contract_panel then
		if self.TR and self.TR.review_text then 
			self:clear_review()
		end
		self._panel:remove(self._contract_panel)
		self._contract_panel = nil
	end
	self:create_contract_box()
end

function ContractBoxGui:create_contract_box()

	--if not managers.network:session() then
		--return
	--end

	--if self._contract_panel then return end

	TeamReview = TeamReview or {}
	TeamReview.contract_box = self
	
	local contact_data = managers.job:current_contact_data()
	local job_data = managers.job:current_job_data()
	local job_chain = managers.job:current_job_chain_data()
	local job_id = managers.job:current_job_id()
	local job_tweak = tweak_data.levels[job_id]
	
	self._contract_panel = self._contract_panel or self._panel:panel({
		name = "contract_box_panel",
		h = 100,
		layer = 0,
		w = self._panel:w() * 0.45 --0.35
	})
	
	self._contract_panel:rect({
		halign = "grow",
		valign = "grow",
		layer = -1,
		color = Color(0.5, 0, 0, 0)
	})
	
	local font = tweak_data.menu.pd2_small_font
	local font_size = tweak_data.menu.pd2_small_font_size
	
	if contact_data then
		if self._contract_text_header then
			self._panel:remove(self._contract_text_header)
		end
		self._contract_text_header = self._panel:text({
			blend_mode = "add",
			text = utf8.to_upper(managers.localization:text(contact_data.name_id) .. ": " .. managers.localization:text(job_data.name_id)),
			h = tweak_data.menu.pd2_medium_font_size,
			font_size = tweak_data.menu.pd2_medium_font_size,
			font = tweak_data.menu.pd2_medium_font,
			color = tweak_data.screen_colors.text
		})
		
		local _, _, tw, th = self._contract_text_header:text_rect()

		self._contract_text_header:set_size(tw, th)
		
		
		self._contract_panel:set_rightbottom(self._panel:w() - 0, self._panel:h() - 60)
		
		self._contract_text_header:set_bottom(self._contract_panel:top())
		self._contract_text_header:set_left(self._contract_panel:left())
		
		self._contract_panel:set_visible(true)
		self._contract_text_header:set_visible(true)
		
		local job_stars = managers.job:current_job_stars()
		local difficulty_stars = managers.job:current_difficulty_stars()
		local diff = tweak_data.difficulty_name_ids[tweak_data.difficulties[difficulty_stars + 2] or 1]

		local job_info_text = managers.localization:to_upper_text(diff)..(Global.game_settings.one_down and ', '..managers.localization:to_upper_text("menu_one_down") or '')
		
		local function ToShortNumber(num)
			local l = num:len()
			
			if l > 6 then
				return num:sub(1, l-6)..'m'
			end
			
			if l > 3 then
				return num:sub(1, l-3)..'k'
			end
			
			return(num)
		end
		
		local contract_visuals = job_data.contract_visuals or {}
		local xp_min = contract_visuals.min_mission_xp and (type(contract_visuals.min_mission_xp) == "table" and contract_visuals.min_mission_xp[difficulty_stars + 1] or contract_visuals.min_mission_xp) or 0
		local xp_max = contract_visuals.max_mission_xp and (type(contract_visuals.max_mission_xp) == "table" and contract_visuals.max_mission_xp[difficulty_stars + 1] or contract_visuals.max_mission_xp) or 0
		local total_xp_min, _ = managers.experience:get_contract_xp_by_stars(job_id, job_stars, difficulty_stars, job_data.professional, #job_chain, {mission_xp = xp_min})
		local total_xp_max, _ = managers.experience:get_contract_xp_by_stars(job_id, job_stars, difficulty_stars, job_data.professional, #job_chain, {mission_xp = xp_max})
		local xp_text_min = ToShortNumber(tostring(math.round(total_xp_min)))
		local xp_text_max = ToShortNumber(tostring(math.round(total_xp_max)))
		local job_xp_text = total_xp_min < total_xp_max and managers.localization:text("menu_number_range", {
			min = xp_text_min,
			max = xp_text_max
		}) or xp_text_min
		
		job_info_text = job_info_text .. ',  ' .. job_xp_text .. ' XP'
		
		local job_ghost_mul = managers.job:get_ghost_bonus() or 0
		if job_ghost_mul ~= 0 then
			local job_ghost = math.round(job_ghost_mul * 100)
			local job_ghost_string = tostring(math.abs(job_ghost))

			if job_ghost == 0 and job_ghost_mul ~= 0 then
				job_ghost_string = string.format("%0.2f", math.abs(job_ghost_mul * 100))
			end

			local text_prefix = job_ghost_mul < 0 and "-" or "+"
			job_info_text = job_info_text .. " (" .. text_prefix .. job_ghost_string .. "%)"
		end
		
		local job_heat = managers.job:current_job_heat() or 0
		local job_heat_mul = managers.job:heat_to_experience_multiplier(job_heat) - 1

		if job_heat_mul ~= 0 then
			job_heat = math.round(job_heat_mul * 100)
			local job_heat_string = tostring(math.abs(job_heat))
			local heat_color = managers.job:current_job_heat_color()

			if job_heat == 0 and job_heat_mul ~= 0 then
				job_heat_string = string.format("%0.2f", math.abs(job_heat_mul * 100))
			end

			local text_prefix = job_heat_mul < 0 and "-" or "+"
			job_info_text = job_info_text .. " (" .. text_prefix .. job_heat_string .. "%)"
		end
		
		local total_payout_min, base_payout_min, risk_payout_min = managers.money:get_contract_money_by_stars(job_stars, difficulty_stars, #job_chain, managers.job:current_job_id(), managers.job:current_level_id())
		local total_payout_max, base_payout_max, risk_payout_max = managers.money:get_contract_money_by_stars(job_stars, difficulty_stars, #job_chain, managers.job:current_job_id(), managers.job:current_level_id(), {
			mandatory_bags_value = contract_visuals.mandatory_bags_value and contract_visuals.mandatory_bags_value[difficulty_stars + 1],
			bonus_bags_value = contract_visuals.bonus_bags_value and contract_visuals.bonus_bags_value[difficulty_stars + 1],
			small_value = contract_visuals.small_value and contract_visuals.small_value[difficulty_stars + 1],
			vehicle_value = contract_visuals.vehicle_value and contract_visuals.vehicle_value[difficulty_stars + 1]
		})
		local payout_text_min = '$'..ToShortNumber(tostring(math.round(total_payout_min)))
		local payout_text_max = '$'..ToShortNumber(tostring(math.round(total_payout_max)))
		local total_payout_text = total_payout_min < total_payout_max and managers.localization:text("menu_number_range", {
			min = payout_text_min,
			max = payout_text_max
		}) or payout_text_min
		
		job_info_text = job_info_text .. ',  ' .. total_payout_text
		
		if managers.mutators:are_mutators_enabled() then
			job_info_text = job_info_text .. ',  ' .. managers.localization:to_upper_text("menu_cn_mutators_active")
		end
		
		self.TR = {}
		self.TR.job_info = self.TR.job_info or self._contract_panel:text({
				name = 'job_info',
				blend_mode = "add",
				x = 5,
				y = 2,
				font = font,
				font_size = font_size,
				text = job_info_text
			})
	end
	
	for i = 1, tweak_data.max_players, 1 do
		local peer = managers.network:session():peer(i)

		if peer then
			local peer_pos = managers.menu_scene:character_screen_position(i)
			local peer_name = peer:name()
			
			if peer_pos then
				self:create_character_text(i, peer_pos.x, peer_pos.y, peer_name)
			end
		end
	end
	
	self._enabled = true
end

local cr_ch_te_orig = ContractBoxGui.create_character_text
function ContractBoxGui:create_character_text(peer_id, x, y, ...)
	if peer_id == 4 or peer_id == 1 then
		y = y - 60
	end
	cr_ch_te_orig(self, peer_id, x, y, ...)
end