function SkirmishContractBoxGui:create_contract_box()
	if not managers.network:session() then
		return
	end

	if self._contract_panel and alive(self._contract_panel) then
		self._panel:remove(self._contract_panel)
	end

	if self._contract_text_header and alive(self._contract_text_header) then
		self._panel:remove(self._contract_text_header)
	end

	if alive(self._panel:child("pro_text")) then
		self._panel:remove(self._panel:child("pro_text"))
	end

	self._contract_panel = nil
	self._contract_text_header = nil
	local contact_data = managers.job:current_contact_data()
	local job_data = managers.job:current_job_data()
	local job_chain = managers.job:current_job_chain_data()
	local job_id = managers.job:current_job_id()
	local job_tweak = tweak_data.levels[job_id]
	self._contract_panel = self._panel:panel({
		name = "contract_box_panel",
		h = 100,
		layer = 0,
		w = self._panel:w() * 0.45
	})

	self._contract_panel:rect({
		halign = "grow",
		valign = "grow",
		layer = -1,
		color = Color(0.5, 0, 0, 0)
	})

	local font = tweak_data.menu.pd2_small_font
	local font_size = tweak_data.menu.pd2_small_font_size
	
	local job_info_text = ''

	if contact_data then
		self._contract_text_header = self._panel:text({
			blend_mode = "add",
			text = managers.localization:to_upper_text(self:_is_weekly() and "menu_weekly_skirmish" or "menu_skirmish")..(job_data and ' ('..managers.localization:text(job_data.name_id)..')' or ''),
			h = tweak_data.menu.pd2_medium_font_size,
			font_size = tweak_data.menu.pd2_medium_font_size,
			font = tweak_data.menu.pd2_medium_font,
			color = tweak_data.screen_colors.text
		})
		local _, _, tw, th = self._contract_text_header:text_rect()

		self._contract_text_header:set_size(tw, th)
		
		local function ToShortNumber(num)
			local l = num:len()
			
			if l > 6 then
				return num:sub(1, l-6)..'m'
			end
			
			if l > 3 then
				return num:sub(1, l-3)..'k'
			end
			
			return(num)
		end

		local job_stars = managers.job:current_job_stars()
		local difficulty_stars = managers.job:current_difficulty_stars()
		local contract_visuals = job_data.contract_visuals or {}
		local xp_min = contract_visuals.min_mission_xp and contract_visuals.min_mission_xp or 0
		local xp_max = contract_visuals.max_mission_xp and contract_visuals.max_mission_xp or 0
		local total_xp_min = managers.experience:get_contract_xp_by_stars(job_id, job_stars, difficulty_stars, job_data.professional, #job_chain, {mission_xp = xp_min})
		local total_xp_max = managers.experience:get_contract_xp_by_stars(job_id, job_stars, difficulty_stars, job_data.professional, #job_chain, {mission_xp = xp_max})
		local xp_text_min = ToShortNumber(tostring(math.round(total_xp_min)))
		local xp_text_max = ToShortNumber(tostring(math.round(total_xp_max)))
		local job_xp_text = total_xp_min < total_xp_max and managers.localization:text("menu_number_range", {
			min = xp_text_min,
			max = xp_text_max
		}) or xp_text_min
		local min_ransom = tostring(tweak_data.skirmish.ransom_amounts[1])
		local max_ransom = tostring(tweak_data.skirmish.ransom_amounts[select(2, managers.skirmish:wave_range())])
		local total_payout_text = min_ransom < max_ransom and managers.localization:text("menu_number_range", {
			min = '$'..ToShortNumber(min_ransom),
			max = '$'..ToShortNumber(max_ransom)
		}) or '$'..ToShortNumber(min_ransom)
		local next_top = 10
		local lines = {}
		local heading_max_width = 0

		local waves_heading = managers.localization:to_upper_text("cn_menu_skirmish_contract_waves_header")
		local waves_value = string.format("%d - %d", managers.skirmish:wave_range())
		job_info_text = job_info_text .. waves_heading .. ' ' .. waves_value

		if self:_is_weekly() then
			local modifiers_heading = managers.localization:to_upper_text("cn_menu_skirmish_contract_modifiers_header")
			local modifiers_value = managers.localization:to_upper_text("cn_menu_contract_mutators_active")

			job_info_text = job_info_text .. ', ' .. modifiers_heading .. ' ' .. modifiers_value
		end

		job_info_text = job_info_text .. ', ' .. job_xp_text .. ' XP'

		job_info_text = job_info_text .. ', ' .. total_payout_text
		
		self.TR = self.TR or {}
		self.TR.job_info = self.TR.job_info or self._contract_panel:text({
				name = 'job_info',
				blend_mode = "add",
				x = 5,
				y = 2,
				font = font,
				font_size = font_size,
				text = job_info_text
			})

	elseif managers.menu:debug_menu_enabled() then
		local debug_start = self._contract_panel:text({
			text = "Use DEBUG START to start your level",
			y = 10,
			wrap = true,
			x = 10,
			word_wrap = true,
			font_size = font_size,
			font = font,
			color = tweak_data.screen_colors.text
		})

		debug_start:grow(-debug_start:x() - 10, debug_start:y() - 10)
	end

	self._contract_panel:set_rightbottom(self._panel:w() - 0, self._panel:h() - 60)

	if self._contract_text_header then
		self._contract_text_header:set_bottom(self._contract_panel:top())
		self._contract_text_header:set_left(self._contract_panel:left())

		local wfs_text = self._panel:child("wfs")

		if wfs_text and not managers.menu:is_pc_controller() then
			wfs_text:set_rightbottom(self._panel:w() - 20, self._contract_text_header:top())
		end
	end

	local wfs = self._panel:child("wfs")

	if wfs then
		self._contract_panel:grow(0, wfs:h() + 5)
		self._contract_panel:move(0, -(wfs:h() + 5))

		if self._contract_text_header then
			self._contract_text_header:move(0, -(wfs:h() + 5))
		end

		wfs:set_world_rightbottom(self._contract_panel:world_right() - 5, self._contract_panel:world_bottom())
	end

	for i = 1, tweak_data.max_players, 1 do
		local peer = managers.network:session():peer(i)

		if peer then
			local peer_pos = managers.menu_scene:character_screen_position(i)
			local peer_name = peer:name()

			if peer_pos then
				self:create_character_text(i, peer_pos.x, peer_pos.y, peer_name)
			end
		end
	end

	self._enabled = true
end